package com.bipt.intelligentapplicationorchestrationservice.mapper;

import com.bipt.intelligentapplicationorchestrationservice.pojo.*;
import org.apache.ibatis.annotations.*;

import java.util.List;

public interface ModelMapper {

    /**
     * 插入模型基本信息
     * @param modelInfo
     */
    @Options(useGeneratedKeys = true, keyProperty = "id")   //能够拿到主键值，并设置给modelInfo对象
    @Insert("insert into model_info(model_name) values(#{modelName})")
    void insertModelInfo(ModelInfo modelInfo);

    /**
     * 插入模型版本信息
     * @param modelVersion
     */
    void insertModelVersion(ModelVersion modelVersion);

    /*
     * 查询模型列表
     */
    List<ModelVO> list();

    /**
     * 根据id查询模型详情
     * @param id  版本表id
     * @return
     */
    ModelVersion selectById(Long id);

    /**
     * 更新模型信息
     * @param dto
     */
    void update(ModelVersionDTO dto);

    /**
     * 删除模型版本
     * @param id  版本表id
     */
    @Delete("delete from model_version where id=#{id}")
    void deleteModelVersion(Long id);

    /**
     * 删除模型
     * @param id  模型id
     */
    @Delete("delete from model_info where id= #{id}")
    void deleteModel(Long id);

    /**
     * 查询模型生命周期
     * @param id  模型版本表id
     *            返回模型生命周期
     */
    @Select("select life_cycle from model_version where id=#{id}")
    String selectLifeCycleById(Long id);

    /**
     * 更新模型生命周期
     * @param id  模型版本表id
     * @param dbValue  数据库中存储的生命周期
     * @return
     */
    @Update("update model_version set life_cycle=#{dbValue} where id=#{id}")
    int updateLifeCycleById(Long id, String dbValue);

    /**
     * 查询数据集列表
     * @return
     */
    @Select("select dataset_id,dataset_name from dataset")
    List<DatasetEntity> listDataset();
}
