package com.bipt.intelligentapplicationorchestrationservice.service.Impl;

import com.bipt.intelligentapplicationorchestrationservice.mapper.AlgorithmInfoMapper;
import com.bipt.intelligentapplicationorchestrationservice.pojo.AlgorithmInfo;
import com.bipt.intelligentapplicationorchestrationservice.service.AlgorithmInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

@Service
public class AlgorithmInfoServiceImpl implements AlgorithmInfoService {

    @Autowired
    private AlgorithmInfoMapper algorithmInfoMapper;

    @Override
    public AlgorithmInfo getById(Long id) {
        return algorithmInfoMapper.selectById(id);
    }

    @Override
    public AlgorithmInfo getByName(String algorithmName) {
        return algorithmInfoMapper.selectByName(algorithmName);
    }

    @Override
    public List<AlgorithmInfo> getByType(String algorithmType) {
        return algorithmInfoMapper.selectByType(algorithmType);
    }

    @Override
    @Transactional
    public boolean update(AlgorithmInfo algorithmInfo) {
        if (!validateAlgorithmInfo(algorithmInfo)) {
            return false;
        }
        return algorithmInfoMapper.update(algorithmInfo) > 0;
    }

    @Override
    @Transactional
    public boolean delete(Long id) {
        return algorithmInfoMapper.deleteById(id) > 0;
    }

    @Override
    public boolean validateAlgorithmInfo(AlgorithmInfo algorithmInfo) {
        if (algorithmInfo == null) {
            return false;
        }
        
        if (!StringUtils.hasText(algorithmInfo.getAlgorithmName()) ||
            !StringUtils.hasText(algorithmInfo.getAlgorithmFile()) ||
            !StringUtils.hasText(algorithmInfo.getAlgorithmType())) {
            return false;
        }
        
        return true;
    }

    /**
     * 算法创建
     * @param algorithmInfo
     */
    @Override
    @Transactional
    public void save(AlgorithmInfo algorithmInfo) {
        String algorithmName = algorithmInfo.getAlgorithmName();
        //查找表里是否有重复的算法，如果有则报错
        AlgorithmInfo duplicateName = algorithmInfoMapper.selectByName(algorithmName);
        if (duplicateName != null){
            throw new RuntimeException("算法已存在，请去修改算法");
        }

        //todo 算法文件分布式存入分布式存储中

        algorithmInfoMapper.insert(algorithmInfo);
    }

    /**
     * 算法执行
     * @param id
     */
    @Override
    public void run(Long id) {
        String description = algorithmInfoMapper.getDescriptionById(id);
        /*//拿到传入的描述，并且用逗号分隔
        String[] commaParts = description.split(",");

        List<String> results = new ArrayList<>();
        for (String part : commaParts) {
            String[] semicolonParts = part.split(";", 2);
            if (semicolonParts.length > 1) {
                results.add(semicolonParts[1]);
            } else {
                throw new RuntimeException("描述格式错误,请修改描述格式");
            }
        }
        String bag = results.getFirst();
        String function = results.get(1);*/
        String file = algorithmInfoMapper.getFileById(id);
        try {
            // 构建命令，将 param 作为参数传递给 Python 脚本
            ProcessBuilder pb = new ProcessBuilder("python", file, description);

            // 设置工作目录（如果 Python 脚本不在当前目录）
            // pb.directory(new File("/path/to/script"));

            // 启动进程
            Process process = pb.start();

            // 获取进程输出
            BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
            String line;
            while ((line = reader.readLine()) != null) {
                System.out.println(line);
            }

            // 获取错误输出
            BufferedReader errorReader = new BufferedReader(new InputStreamReader(process.getErrorStream()));
            String errorLine;
            while ((errorLine = errorReader.readLine()) != null) {
                System.err.println("Error: " + errorLine);
            }
            // 等待进程完成
            int exitCode = process.waitFor();
            System.out.println("Exit Code: " + exitCode);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
} 